﻿////////////////////////////////////////////////////////////////////////////////
//
// AudioscrobblerReader
//   RecenttracksXML.as
//   Rev. 1.1 (2005/9/25)
// 
// by nomura masayuki
// http://www.a3size.com/AudioscrobblerReader/
//
////////////////////////////////////////////////////////////////////////////////
//
// RecenttracksXMLクラス
// RecenttracksXMLのモデル
//
////////////////////////////////////////////////////////////////////////////////
import com.a3size.audioscrobblerReader.*;

class com.a3size.audioscrobblerReader.RecenttracksXML extends XML
{
	// XML定義
	static var XMLNODENAME:String = "recenttracks";
	static var ITEMNODENAME:String = "track";
	static var SONGTITLENODENAME:String = "name";
	static var SONGDATENODENAME:String = "date";
	static var ARTISTNODENAME:String = "artist";

	// アイテム
	private var itemData:Array;
	// 親コントローラーの定義
	private var parentControler:XMLControler;
	
	// その他のXMLの情報
	private var xmlDateTime:Date;

	// コンストラクタ
	function RecenttracksXML(controler:XMLControler)
	{
		super();
		this.parentControler = controler;
		itemData = new Array();
		xmlDateTime = null;
	}
	
	function resetXML():Void
	{
		this.firstChild.removeNode();
		itemData = new Array();
		xmlDateTime = null;
	}

	// ロード時イベント
	function onLoad( success:Boolean ): Void
	{
		// ダウンロードの成否、パースの成否をコントローラに通知
		if (success && readSongDataArray())
			parentControler.loaded(true);
		else
			parentControler.loaded(false);
	}

	// XMLファイルからデータを取り出す
	function readSongDataArray():Boolean
	{
		var _xml:XMLNode = this.lastChild;
		// recenttracksノードがない場合、パースを実行しない
		if (_xml.nodeName != XMLNODENAME) return false;

		parseXMLItems(_xml);
		return true;
	}

	// メンバ(get/set)
	function get item():Array
	{
		return itemData;
	}
	function get xmlDate():Date
	{
		return this.xmlDateTime;
	}
	function set xmlDate(date:Date):Void
	{
		this.xmlDateTime = date;
	}

	// アイテム項目のパース
	private function parseXMLItems(node_xml:XMLNode):Void
	{
		if (node_xml.nodeName == ITEMNODENAME)
		{
			var songItem = getItem(node_xml);
			itemData.push(songItem);
		}
		else
		{
			if (node_xml.hasChildNodes())
			{
				var nodeArray:Array = node_xml.childNodes;
				for (var i:Number = 0; i < nodeArray.length; i++)
				{
					parseXMLItems(nodeArray[i]);
				}
			}
		}
	}

	// アイテム項目の内容のパース
	private function getItem(node_xml:XMLNode):SongItem
	{
		var songItem:SongItem = new SongItem();
		
		songItem.song = getSongTitle(node_xml.childNodes);
		songItem.artist = getSongArtist(node_xml.childNodes);
		songItem.date = getSongDate(node_xml.childNodes);
		
		return songItem;
	}

	// 曲名のパース
	private function getSongTitle(node_array:Array):String
	{
		var title_str:String = "";
		var i:Number = 0;
		while ((i < node_array.length) && (title_str.length == 0))
		{
			if (node_array[i].nodeName == SONGTITLENODENAME)
				title_str = node_array[i].firstChild.nodeValue;
			i++;
		}
		return title_str;
	}
	
	// アーティスト名のパース
	private function getSongArtist(node_array:Array):String
	{
		var title_str:String = "";
		var i:Number = 0;
		while ((i < node_array.length) && (title_str.length == 0))
		{
			if (node_array[i].nodeName == ARTISTNODENAME)
				title_str = node_array[i].firstChild.nodeValue;
			i++;
		}
		return title_str;
	}

	// 日付のパース
	private function getSongDate(node_array:Array):Date
	{
		var date_str:String = "";
		var i:Number = 0;
		while ((i < node_array.length) && (date_str.length == 0))
		{
			if (node_array[i].nodeName == SONGDATENODENAME)
				date_str = node_array[i].attributes["uts"];
			i++;
		}
		
		var date:Date = new Date();
		if (date_str.length > 0)
		{
			date.setTime(Number(date_str) * 1000);
		}
		return date;
	}
}